@@ -24,7 +24,7 @@ class C:
     i: bytes
     j: E
     k: int
-    l: float
+    l: float  # noqa: E741
     m: str
     n: bytes
     o: E
@@ -60,32 +60,32 @@ def test_unstructure_attrs_primitives(benchmark, converter_cls, unstructure_stra
             1,
             1.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.ONE,
             2,
             2.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.TWO,
             3,
             3.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.ONE,
             4,
             4.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.TWO,
             5,
             5.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.ONE,
             6,
             6.0,
             "a small string",
-            "test".encode(),
+            b"test",
             E.TWO,
         ),
     )
@@ -104,32 +104,32 @@ def test_structure_attrs_primitives(benchmark, converter_cls, unstructure_strat)
         1,
         1.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.ONE,
         2,
         2.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.TWO,
         3,
         3.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.ONE,
         4,
         4.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.TWO,
         5,
         5.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.ONE,
         6,
         6.0,
         "a small string",
-        "test".encode(),
+        b"test",
         E.TWO,
     )
 
