@@ -1,7 +1,9 @@
 import os
+from typing import Literal
 
 from hypothesis import HealthCheck, settings
 from hypothesis.strategies import just, one_of
+from typing_extensions import TypeAlias
 
 from cattrs import UnstructureStrategy
 
@@ -9,7 +11,9 @@ settings.register_profile(
     "CI", settings(suppress_health_check=[HealthCheck.too_slow]), deadline=None
 )
 
-if "CI" in os.environ:
+if "CI" in os.environ:  # pragma: nocover
     settings.load_profile("CI")
 
 unstructure_strats = one_of(just(s) for s in UnstructureStrategy)
+
+FeatureFlag: TypeAlias = Literal["always", "never", "sometimes"]
