@@ -1,15 +1,21 @@
+import functools
 import typing
 from copy import deepcopy
+from dataclasses import dataclass
 from functools import partial
-from typing import List, Tuple
+from typing import Any
 
 import pytest
 from attrs import define
 
 from cattrs import Converter, override
-from cattrs.errors import ClassValidationError
+from cattrs.errors import ClassValidationError, StructureHandlerNotFoundError
 from cattrs.strategies import configure_tagged_union, include_subclasses
 
+from .._compat import is_py311_plus
+
+T = typing.TypeVar("T")
+
 
 @define
 class Parent:
@@ -148,7 +154,7 @@ def conv_w_subclasses(request):
     "struct_unstruct", IDS_TO_STRUCT_UNSTRUCT.values(), ids=IDS_TO_STRUCT_UNSTRUCT
 )
 def test_structuring_with_inheritance(
-    conv_w_subclasses: Tuple[Converter, bool], struct_unstruct
+    conv_w_subclasses: tuple[Converter, bool], struct_unstruct
 ) -> None:
     structured, unstructured = struct_unstruct
 
@@ -219,7 +225,7 @@ def test_circular_reference(conv_w_subclasses):
     "struct_unstruct", IDS_TO_STRUCT_UNSTRUCT.values(), ids=IDS_TO_STRUCT_UNSTRUCT
 )
 def test_unstructuring_with_inheritance(
-    conv_w_subclasses: Tuple[Converter, bool], struct_unstruct
+    conv_w_subclasses: tuple[Converter, bool], struct_unstruct
 ):
     structured, unstructured = struct_unstruct
     converter, included_subclasses_param = conv_w_subclasses
@@ -389,5 +395,117 @@ def test_cycles_classes_2(genconverter: Converter):
                 "_type": "Derived",
             }
         ],
-        List[A],
+        list[A],
     ) == [Derived(9, Derived(99, A(999)))]
+
+
+def test_unsupported_class(genconverter: Converter):
+    """Non-attrs/dataclass classes raise proper errors."""
+
+    class NewParent:
+        """Not an attrs class."""
+
+        a: int
+
+    @define
+    class NewChild(NewParent):
+        pass
+
+    @define
+    class NewChild2(NewParent):
+        pass
+
+    genconverter.register_structure_hook(NewParent, lambda v, _: NewParent(v))
+
+    with pytest.raises(StructureHandlerNotFoundError):
+        include_subclasses(NewParent, genconverter)
+
+
+def test_parents_with_generics(genconverter: Converter):
+    """Ensure proper handling of generic parents #648."""
+
+    @define
+    class GenericParent(typing.Generic[T]):
+        p: T
+
+    @define
+    class Child1G(GenericParent[str]):
+        c: str
+
+    include_subclasses(GenericParent[str], genconverter)
+
+    assert genconverter.structure({"p": 5, "c": 5}, GenericParent[str]) == Child1G(
+        "5", "5"
+    )
+
+
+def test_parents_with_generics_tagged_union(genconverter: Converter):
+    """Ensure proper handling of generic parents with configure_tagged_union, #682."""
+
+    @define
+    class GenericParent(typing.Generic[T]):
+        p: T
+
+    @define
+    class Child1G(GenericParent[str]):
+        c: str
+
+    @define
+    class Child2G(GenericParent[int]):
+        c: str
+
+    union_strategy = functools.partial(
+        configure_tagged_union,
+        tag_generator=lambda cl: (typing.get_origin(cl) or cl).__name__,
+    )
+    include_subclasses(GenericParent[Any], genconverter, union_strategy=union_strategy)
+
+    assert genconverter.unstructure(Child1G("5", "5")) == {
+        "p": "5",
+        "c": "5",
+        "_type": "Child1G",
+    }
+    assert genconverter.unstructure(Child2G(1, "5")) == {
+        "p": 1,
+        "c": "5",
+        "_type": "Child2G",
+    }
+    assert genconverter.structure(
+        {"p": "5", "c": "5", "_type": "Child1G"}, GenericParent[Any]
+    ) == Child1G("5", "5")
+    assert genconverter.structure(
+        {"p": 1, "c": "5", "_type": "Child2G"}, GenericParent[Any]
+    ) == Child2G(1, "5")
+
+
+def test_dataclasses(genconverter: Converter):
+    """Dict dataclasses work."""
+
+    @dataclass
+    class ParentDC:
+        a: int
+
+    @dataclass
+    class ChildDC1(ParentDC):
+        b: str
+
+    include_subclasses(ParentDC, genconverter)
+
+    assert genconverter.structure({"a": 1, "b": "a"}, ParentDC) == ChildDC1(1, "a")
+
+
+@pytest.mark.skipif(not is_py311_plus, reason="slotted dataclasses supported on 3.11+")
+def test_dataclasses_slots(genconverter: Converter):
+    """Slotted dataclasses work."""
+
+    @dataclass(slots=True)
+    class ParentDC:
+        a: int
+
+    @dataclass(slots=True)
+    class ChildDC1(ParentDC):
+        b: str
+
+    include_subclasses(ParentDC, genconverter)
+
+    assert genconverter.structure({"a": 1, "b": "a"}, ParentDC) == ChildDC1(1, "a")
