@@ -1,6 +1,7 @@
 """Tests for generated dict functions."""
 
-from typing import Dict, Type
+from math import ceil
+from typing import Annotated, Dict, Literal, Type, Union
 
 import pytest
 from attrs import NOTHING, Factory, define, field, frozen
@@ -12,12 +13,12 @@ from cattrs._compat import adapted_fields, fields
 from cattrs.errors import ClassValidationError, ForbiddenExtraKeysError
 from cattrs.gen import make_dict_structure_fn, make_dict_unstructure_fn, override
 
-from ._compat import is_py39_plus
+from .helpers import assert_only_unstructured
 from .typed import nested_typed_classes, simple_typed_classes, simple_typed_dataclasses
 from .untyped import nested_classes, simple_classes
 
 
-@given(nested_classes | simple_classes())
+@given(nested_classes() | simple_classes())
 def test_unmodified_generated_unstructuring(cl_and_vals):
     converter = BaseConverter()
     cl, vals, kwargs = cl_and_vals
@@ -34,7 +35,7 @@ def test_unmodified_generated_unstructuring(cl_and_vals):
     assert res_expected == res_actual
 
 
-@given(nested_classes | simple_classes())
+@given(nested_classes() | simple_classes())
 def test_nodefs_generated_unstructuring(cl_and_vals):
     """Test omitting default values on a per-attribute basis."""
     converter = BaseConverter()
@@ -62,7 +63,9 @@ def test_nodefs_generated_unstructuring(cl_and_vals):
         assert attr.name not in res
 
 
-@given(one_of(just(BaseConverter), just(Converter)), nested_classes | simple_classes())
+@given(
+    one_of(just(BaseConverter), just(Converter)), nested_classes() | simple_classes()
+)
 def test_nodefs_generated_unstructuring_cl(
     converter_cls: Type[BaseConverter], cl_and_vals
 ):
@@ -106,7 +109,9 @@ def test_nodefs_generated_unstructuring_cl(
 
 @given(
     one_of(just(BaseConverter), just(Converter)),
-    nested_classes | simple_classes() | simple_typed_dataclasses(),
+    nested_classes()
+    | simple_classes(min_attrs=1)
+    | simple_typed_dataclasses(min_attrs=1),
 )
 def test_individual_overrides(converter_cls, cl_and_vals):
     """
@@ -116,7 +121,9 @@ def test_individual_overrides(converter_cls, cl_and_vals):
     converter = converter_cls()
     cl, vals, kwargs = cl_and_vals
 
-    for attr in adapted_fields(cl):
+    fields = adapted_fields(cl)
+
+    for attr in fields:
         if attr.default is not NOTHING:
             break
     else:
@@ -137,10 +144,9 @@ def test_individual_overrides(converter_cls, cl_and_vals):
     inst = cl(*vals, **kwargs)
 
     res = converter.unstructure(inst)
-    assert "Hyp" not in repr(res)
-    assert "Factory" not in repr(res)
+    assert_only_unstructured(res)
 
-    for attr, val in zip(adapted_fields(cl), vals):
+    for attr, val in zip(fields, vals):
         if attr.name == chosen_name:
             assert attr.name in res
         elif attr.default is not NOTHING:
@@ -160,9 +166,9 @@ def test_individual_overrides(converter_cls, cl_and_vals):
 
 
 @given(
-    cl_and_vals=nested_typed_classes()
-    | simple_typed_classes()
-    | simple_typed_dataclasses(),
+    cl_and_vals=nested_typed_classes(allow_nan=False)
+    | simple_typed_classes(allow_nan=False)
+    | simple_typed_dataclasses(allow_nan=False),
     dv=...,
 )
 def test_unmodified_generated_structuring(cl_and_vals, dv: bool):
@@ -175,7 +181,7 @@ def test_unmodified_generated_structuring(cl_and_vals, dv: bool):
 
     unstructured = converter.unstructure(inst)
 
-    assert "Hyp" not in repr(unstructured)
+    assert_only_unstructured(unstructured)
 
     converter.register_structure_hook(cl, fn)
 
@@ -185,7 +191,9 @@ def test_unmodified_generated_structuring(cl_and_vals, dv: bool):
 
 
 @given(
-    simple_typed_classes(min_attrs=1) | simple_typed_dataclasses(min_attrs=1), data()
+    simple_typed_classes(min_attrs=1, allow_nan=False)
+    | simple_typed_dataclasses(min_attrs=1, allow_nan=False),
+    data(),
 )
 def test_renaming(cl_and_vals, data):
     converter = Converter()
@@ -311,17 +319,15 @@ def test_omitting_structure(detailed_validation: bool):
     assert not hasattr(structured, "b")
 
 
-@pytest.mark.skipif(not is_py39_plus, reason="literals and annotated are 3.9+")
 def test_type_names_with_quotes():
     """Types with quote characters in their reprs should work."""
-    from typing import Annotated, Literal, Union
 
     converter = Converter()
 
     assert converter.structure({1: 1}, Dict[Annotated[int, "'"], int]) == {1: 1}
 
     converter.register_structure_hook_func(
-        lambda t: t is Union[Literal["a", 2, 3], Literal[4]], lambda v, _: v
+        lambda t: t == Union[Literal["a", 2, 3], Literal[4]], lambda v, _: v
     )
     assert converter.structure(
         {2: "a"}, Dict[Union[Literal["a", 2, 3], Literal[4]], str]
@@ -330,12 +336,12 @@ def test_type_names_with_quotes():
 
 def test_overriding_struct_hook(converter: BaseConverter) -> None:
     """Overriding structure hooks works."""
-    from math import ceil
 
     @define
     class A:
         a: int
         b: str
+        c: int = 0
 
     converter.register_structure_hook(
         A,
@@ -343,11 +349,12 @@ def test_overriding_struct_hook(converter: BaseConverter) -> None:
             A,
             converter,
             a=override(struct_hook=lambda v, _: ceil(v)),
+            c=override(struct_hook=lambda v, _: ceil(v)),
             _cattrs_detailed_validation=converter.detailed_validation,
         ),
     )
 
-    assert converter.structure({"a": 0.5, "b": 1}, A) == A(1, "1")
+    assert converter.structure({"a": 0.5, "b": 1, "c": 0.5}, A) == A(1, "1", 1)
 
 
 def test_overriding_unstruct_hook(converter: BaseConverter) -> None:
@@ -556,6 +563,7 @@ def test_init_false_no_structure_hook(converter: BaseConverter):
     @define
     class A:
         a: int = field(converter=int, init=False)
+        b: int = field(converter=int, init=False, default=5)
 
     converter.register_structure_hook(
         A,
@@ -634,8 +642,8 @@ def test_detailed_validation_from_converter(converter: BaseConverter):
             converter.structure({"a": "a"}, A)
 
 
-@given(prefer=...)
-def test_prefer_converters_from_converter(prefer: bool):
+@given(prefer=..., dv=...)
+def test_prefer_converters_from_converter(prefer: bool, dv: bool):
     """
     `prefer_attrs_converters` is taken from the converter by default.
     """
@@ -643,13 +651,17 @@ def test_prefer_converters_from_converter(prefer: bool):
     @define
     class A:
         a: int = field(converter=lambda x: x + 1)
+        b: int = field(converter=lambda x: x + 1, default=5)
 
     converter = BaseConverter(prefer_attrib_converters=prefer)
     converter.register_structure_hook(int, lambda x, _: x + 1)
-    converter.register_structure_hook(A, make_dict_structure_fn(A, converter))
+    converter.register_structure_hook(
+        A, make_dict_structure_fn(A, converter, _cattrs_detailed_validation=dv)
+    )
 
     if prefer:
-        assert converter.structure({"a": 1}, A).a == 2
+        assert converter.structure({"a": 1, "b": 2}, A).a == 2
+        assert converter.structure({"a": 1, "b": 2}, A).b == 3
     else:
         assert converter.structure({"a": 1}, A).a == 3
 
