@@ -1,7 +1,7 @@
 """Tests for TypedDict un/structuring."""
 
 from datetime import datetime, timezone
-from typing import Dict, Generic, NewType, Set, Tuple, TypedDict, TypeVar
+from typing import Generic, NewType, TypedDict, TypeVar
 
 import pytest
 from attrs import NOTHING
@@ -10,7 +10,7 @@ from hypothesis.strategies import booleans
 from pytest import raises
 from typing_extensions import NotRequired, Required
 
-from cattrs import BaseConverter, Converter
+from cattrs import BaseConverter, Converter, transform_error
 from cattrs._compat import ExtensionsTypedDict, get_notrequired_base, is_generic
 from cattrs.errors import (
     ClassValidationError,
@@ -25,14 +25,23 @@ from cattrs.gen.typeddicts import (
     make_dict_unstructure_fn,
 )
 
-from ._compat import is_py38, is_py39, is_py310, is_py311_plus
+from ._compat import is_py311_plus
 from .typeddicts import (
+    gen_typeddict_attr_names,
     generic_typeddicts,
     simple_typeddicts,
     simple_typeddicts_with_extra_keys,
 )
 
 
+def test_gen_attr_names():
+    """We can generate a lot of attribute names."""
+    assert len(list(gen_typeddict_attr_names())) == 697
+
+    # No duplicates!
+    assert len(list(gen_typeddict_attr_names())) == len(set(gen_typeddict_attr_names()))
+
+
 def mk_converter(detailed_validation: bool = True) -> Converter:
     """We can't use function-scoped fixtures with Hypothesis strats."""
     c = Converter(detailed_validation=detailed_validation)
@@ -60,7 +69,7 @@ def get_annot(t) -> dict:
                         NotRequired[param_to_args[nrb]] if nrb in param_to_args else v
                     )
                 else:
-                    res[k] = param_to_args[v] if v in param_to_args else v
+                    res[k] = param_to_args.get(v, v)
             return res
 
         # Origin is `None`, so this is a subclass for a generic typeddict.
@@ -72,12 +81,12 @@ def get_annot(t) -> dict:
                     NotRequired[mapping[nrb.__name__]] if nrb.__name__ in mapping else v
                 )
             else:
-                res[k] = mapping[v.__name__] if v.__name__ in mapping else v
+                res[k] = mapping.get(v.__name__, v)
         return res
     return get_annots(t)
 
 
-@given(simple_typeddicts(typeddict_cls=None if not is_py38 else ExtensionsTypedDict))
+@given(simple_typeddicts())
 def test_simple_roundtrip(cls_and_instance) -> None:
     """Round-trips for simple classes work."""
     c = mk_converter()
@@ -97,12 +106,7 @@ def test_simple_roundtrip(cls_and_instance) -> None:
     assert restructured == instance
 
 
-@given(
-    simple_typeddicts(
-        total=False, typeddict_cls=None if not is_py38 else ExtensionsTypedDict
-    ),
-    booleans(),
-)
+@given(simple_typeddicts(total=False), booleans())
 def test_simple_nontotal(cls_and_instance, detailed_validation: bool) -> None:
     """Non-total dicts work."""
     c = mk_converter(detailed_validation=detailed_validation)
@@ -122,7 +126,7 @@ def test_simple_nontotal(cls_and_instance, detailed_validation: bool) -> None:
     assert restructured == instance
 
 
-@given(simple_typeddicts(typeddict_cls=None if not is_py38 else ExtensionsTypedDict))
+@given(simple_typeddicts())
 def test_int_override(cls_and_instance) -> None:
     """Overriding a base unstructure handler should work."""
     cls, instance = cls_and_instance
@@ -138,14 +142,9 @@ def test_int_override(cls_and_instance) -> None:
     assert unstructured == instance
 
 
-@given(
-    simple_typeddicts_with_extra_keys(
-        typeddict_cls=None if not is_py38 else ExtensionsTypedDict
-    ),
-    booleans(),
-)
+@given(simple_typeddicts_with_extra_keys(), booleans())
 def test_extra_keys(
-    cls_instance_extra: Tuple[type, Dict, Set[str]], detailed_validation: bool
+    cls_instance_extra: tuple[type, dict, set[str]], detailed_validation: bool
 ) -> None:
     """Extra keys are preserved."""
     cls, instance, extra = cls_instance_extra
@@ -167,7 +166,7 @@ def test_extra_keys(
 @pytest.mark.skipif(not is_py311_plus, reason="3.11+ only")
 @given(generic_typeddicts(total=True), booleans())
 def test_generics(
-    cls_and_instance: Tuple[type, Dict], detailed_validation: bool
+    cls_and_instance: tuple[type, dict], detailed_validation: bool
 ) -> None:
     """Generic TypedDicts work."""
     c = mk_converter(detailed_validation=detailed_validation)
@@ -231,7 +230,7 @@ def test_deep_generics(detailed_validation: bool):
 
 @given(simple_typeddicts(total=True, not_required=True), booleans())
 def test_not_required(
-    cls_and_instance: Tuple[type, Dict], detailed_validation: bool
+    cls_and_instance: tuple[type, dict], detailed_validation: bool
 ) -> None:
     """NotRequired[] keys are handled."""
     c = mk_converter(detailed_validation=detailed_validation)
@@ -243,16 +242,9 @@ def test_not_required(
     assert restructured == instance
 
 
-@given(
-    simple_typeddicts(
-        total=False,
-        not_required=True,
-        typeddict_cls=None if not is_py38 else ExtensionsTypedDict,
-    ),
-    booleans(),
-)
+@given(simple_typeddicts(total=False, not_required=True), booleans())
 def test_required(
-    cls_and_instance: Tuple[type, Dict], detailed_validation: bool
+    cls_and_instance: tuple[type, dict], detailed_validation: bool
 ) -> None:
     """Required[] keys are handled."""
     c = mk_converter(detailed_validation=detailed_validation)
@@ -264,9 +256,9 @@ def test_required(
     assert restructured == instance
 
 
-@pytest.mark.skipif(is_py39 or is_py310, reason="Sigh")
+@pytest.mark.skipif(not is_py311_plus, reason="Sigh")
 def test_required_keys() -> None:
-    """We don't support the full gamut of functionality on 3.8.
+    """We don't support the full gamut of functionality on 3.9 and 3.10.
 
     When using `typing.TypedDict` we have only partial functionality;
     this test tests only a subset of this.
@@ -287,7 +279,7 @@ def test_required_keys() -> None:
 
 
 @given(simple_typeddicts(min_attrs=1, total=True), booleans())
-def test_omit(cls_and_instance: Tuple[type, Dict], detailed_validation: bool) -> None:
+def test_omit(cls_and_instance: tuple[type, dict], detailed_validation: bool) -> None:
     """`override(omit=True)` works."""
     c = mk_converter(detailed_validation=detailed_validation)
 
@@ -329,7 +321,7 @@ def test_omit(cls_and_instance: Tuple[type, Dict], detailed_validation: bool) ->
 
 
 @given(simple_typeddicts(min_attrs=1, total=True, not_required=True), booleans())
-def test_rename(cls_and_instance: Tuple[type, Dict], detailed_validation: bool) -> None:
+def test_rename(cls_and_instance: tuple[type, dict], detailed_validation: bool) -> None:
     """`override(rename=...)` works."""
     c = mk_converter(detailed_validation=detailed_validation)
 
@@ -355,7 +347,7 @@ def test_rename(cls_and_instance: Tuple[type, Dict], detailed_validation: bool)
 
 @given(simple_typeddicts(total=True), booleans())
 def test_forbid_extra_keys(
-    cls_and_instance: Tuple[type, Dict], detailed_validation: bool
+    cls_and_instance: tuple[type, dict], detailed_validation: bool
 ) -> None:
     """Extra keys can be forbidden."""
     c = mk_converter(detailed_validation)
@@ -390,13 +382,7 @@ def test_forbid_extra_keys(
         assert repr(ctx.value) == repr(
             ClassValidationError(
                 f"While structuring {cls.__name__}",
-                [
-                    ForbiddenExtraKeysError(
-                        f"Extra fields in constructor for {cls.__name__}: test",
-                        cls,
-                        {"test"},
-                    )
-                ],
+                [ForbiddenExtraKeysError("", cls, {"test"})],
                 cls,
             )
         )
@@ -517,3 +503,21 @@ def test_override_entire_hooks(converter: BaseConverter):
 
     assert converter.unstructure({"a": 10, "b": 10}, A) == {"a": 1, "b": 2}
     assert converter.structure({"a": 10, "b": 10}, A) == {"a": 1, "b": 2}
+
+
+def test_nondict_input():
+    """Trying to structure typeddict from a non-dict raises the proper exception."""
+    converter = Converter(detailed_validation=True)
+    with raises(ClassValidationError) as exc:
+        converter.structure(1, TypedDictA)
+
+    assert transform_error(exc.value) == [
+        "invalid type (expected a mapping, not int) @ $"
+    ]
+
+    with raises(ClassValidationError) as exc:
+        converter.structure([1], TypedDictA)
+
+    assert transform_error(exc.value) == [
+        "invalid type (expected a mapping, not list) @ $"
+    ]
