@@ -1,7 +1,5 @@
 """Strategies for attributes with types and classes using them."""
 
-import sys
-from collections import OrderedDict
 from collections.abc import MutableSequence as AbcMutableSequence
 from collections.abc import MutableSet as AbcMutableSet
 from collections.abc import Sequence as AbcSequence
@@ -10,6 +8,7 @@ from dataclasses import field as dc_field
 from dataclasses import make_dataclass
 from functools import partial
 from pathlib import Path
+from string import ascii_lowercase
 from typing import (
     Any,
     Dict,
@@ -19,7 +18,6 @@ from typing import (
     MutableSequence,
     MutableSet,
     NewType,
-    Optional,
     Sequence,
     Set,
     Tuple,
@@ -28,7 +26,7 @@ from typing import (
 )
 
 from attr._make import _CountingAttr
-from attrs import NOTHING, Factory, field, frozen
+from attrs import NOTHING, AttrsInstance, Factory, field, frozen
 from hypothesis import note
 from hypothesis.strategies import (
     DrawFn,
@@ -50,25 +48,25 @@ from hypothesis.strategies import (
     tuples,
 )
 
+from . import FeatureFlag
 from .untyped import gen_attr_names, make_class
 
-is_39_or_later = sys.version_info[:2] >= (3, 9)
 PosArg = Any
-PosArgs = Tuple[PosArg]
-KwArgs = Dict[str, Any]
+PosArgs = tuple[PosArg]
+KwArgs = dict[str, Any]
 T = TypeVar("T")
 
 
 def simple_typed_classes(
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     min_attrs=0,
     frozen=False,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
     newtypes=True,
     text_codec: str = "utf8",
     allow_infinity=None,
-    allow_nan=None,
-) -> SearchStrategy[Tuple[Type, PosArgs, KwArgs]]:
+    allow_nan=True,
+) -> SearchStrategy[tuple[type, PosArgs, KwArgs]]:
     """Yield tuples of (class, values)."""
     return lists_of_typed_attrs(
         defaults,
@@ -82,7 +80,13 @@ def simple_typed_classes(
     ).flatmap(partial(_create_hyp_class, frozen=frozen))
 
 
-def simple_typed_dataclasses(defaults=None, min_attrs=0, frozen=False, newtypes=True):
+def simple_typed_dataclasses(
+    defaults: FeatureFlag = "sometimes",
+    min_attrs=0,
+    frozen=False,
+    newtypes=True,
+    allow_nan=True,
+):
     """Yield tuples of (class, values)."""
     return lists_of_typed_attrs(
         defaults,
@@ -90,29 +94,38 @@ def simple_typed_dataclasses(defaults=None, min_attrs=0, frozen=False, newtypes=
         for_frozen=frozen,
         allow_mutable_defaults=False,
         newtypes=newtypes,
+        allow_nan=allow_nan,
     ).flatmap(partial(_create_dataclass, frozen=frozen))
 
 
 def simple_typed_classes_and_strats(
-    defaults=None, min_attrs=0, kw_only=None, newtypes=True
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
+    defaults: FeatureFlag = "sometimes",
+    min_attrs=0,
+    kw_only: FeatureFlag = "sometimes",
+    newtypes=True,
+    allow_nan=True,
+) -> SearchStrategy[tuple[type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
     """Yield tuples of (class, (strategies))."""
     return lists_of_typed_attrs(
-        defaults, min_size=min_attrs, kw_only=kw_only, newtypes=newtypes
+        defaults,
+        min_size=min_attrs,
+        kw_only=kw_only,
+        newtypes=newtypes,
+        allow_nan=allow_nan,
     ).flatmap(_create_hyp_class_and_strat)
 
 
 def lists_of_typed_attrs(
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     min_size=0,
     for_frozen=False,
     allow_mutable_defaults=True,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
     newtypes=True,
     text_codec="utf8",
     allow_infinity=None,
-    allow_nan=None,
-) -> SearchStrategy[List[Tuple[_CountingAttr, SearchStrategy[PosArg]]]]:
+    allow_nan=True,
+) -> SearchStrategy[list[tuple[_CountingAttr, SearchStrategy[PosArg]]]]:
     # Python functions support up to 255 arguments.
     return lists(
         simple_typed_attrs(
@@ -135,97 +148,49 @@ def lists_of_typed_attrs(
 
 
 def simple_typed_attrs(
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     for_frozen=False,
     allow_mutable_defaults=True,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
     newtypes=True,
     text_codec="utf8",
     allow_infinity=None,
-    allow_nan=None,
-) -> SearchStrategy[Tuple[_CountingAttr, SearchStrategy[PosArgs]]]:
-    if not is_39_or_later:
+    allow_nan=True,
+) -> SearchStrategy[tuple[_CountingAttr, SearchStrategy[PosArgs]]]:
+    res = (
+        any_typed_attrs(defaults, kw_only)
+        | int_typed_attrs(defaults, kw_only)
+        | str_typed_attrs(defaults, kw_only, text_codec)
+        | float_typed_attrs(defaults, kw_only, allow_infinity, allow_nan)
+        | frozenset_typed_attrs(defaults, kw_only=kw_only)
+        | seq_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
+        | homo_tuple_typed_attrs(defaults, kw_only=kw_only)
+        | path_typed_attrs(defaults, kw_only=kw_only)
+    )
+    if newtypes:
         res = (
-            any_typed_attrs(defaults, kw_only)
-            | int_typed_attrs(defaults, kw_only)
-            | str_typed_attrs(defaults, kw_only, text_codec)
-            | float_typed_attrs(defaults, kw_only, allow_infinity, allow_nan)
-            | frozenset_typed_attrs(defaults, legacy_types_only=True, kw_only=kw_only)
-            | homo_tuple_typed_attrs(defaults, legacy_types_only=True, kw_only=kw_only)
-            | path_typed_attrs(defaults, kw_only=kw_only)
+            res
+            | newtype_int_typed_attrs(defaults, kw_only)
+            | newtype_attrs_typed_attrs(defaults, kw_only)
         )
-        if newtypes:
-            res = (
-                res
-                | newtype_int_typed_attrs(defaults, kw_only)
-                | newtype_attrs_typed_attrs(defaults, kw_only)
-            )
-        if not for_frozen:
-            res = (
-                res
-                | dict_typed_attrs(defaults, allow_mutable_defaults, kw_only)
-                | mutable_seq_typed_attrs(
-                    defaults,
-                    allow_mutable_defaults,
-                    legacy_types_only=True,
-                    kw_only=kw_only,
-                )
-                | seq_typed_attrs(
-                    defaults,
-                    allow_mutable_defaults,
-                    legacy_types_only=True,
-                    kw_only=kw_only,
-                )
-                | list_typed_attrs(
-                    defaults,
-                    allow_mutable_defaults,
-                    legacy_types_only=True,
-                    kw_only=kw_only,
-                )
-                | set_typed_attrs(
-                    defaults,
-                    allow_mutable_defaults,
-                    legacy_types_only=True,
-                    kw_only=kw_only,
-                )
-            )
-    else:
-        res = (
-            any_typed_attrs(defaults, kw_only)
-            | int_typed_attrs(defaults, kw_only)
-            | str_typed_attrs(defaults, kw_only, text_codec)
-            | float_typed_attrs(defaults, kw_only, allow_infinity, allow_nan)
-            | frozenset_typed_attrs(defaults, kw_only=kw_only)
-            | homo_tuple_typed_attrs(defaults, kw_only=kw_only)
-            | path_typed_attrs(defaults, kw_only=kw_only)
-        )
-        if newtypes:
-            res = (
-                res
-                | newtype_int_typed_attrs(defaults, kw_only)
-                | newtype_attrs_typed_attrs(defaults, kw_only)
-            )
 
-        if not for_frozen:
-            res = (
-                res
-                | dict_typed_attrs(defaults, allow_mutable_defaults, kw_only)
-                | new_dict_typed_attrs(defaults, allow_mutable_defaults, kw_only)
-                | set_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
-                | list_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
-                | mutable_seq_typed_attrs(
-                    defaults, allow_mutable_defaults, kw_only=kw_only
-                )
-                | seq_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
-            )
+    if not for_frozen:
+        res = (
+            res
+            | dict_typed_attrs(defaults, allow_mutable_defaults, kw_only)
+            | new_dict_typed_attrs(defaults, allow_mutable_defaults, kw_only)
+            | set_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
+            | list_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
+            | mutable_seq_typed_attrs(defaults, allow_mutable_defaults, kw_only=kw_only)
+        )
 
     return res
 
 
 def _create_hyp_class(
-    attrs_and_strategy: List[Tuple[_CountingAttr, SearchStrategy[PosArgs]]],
+    attrs_and_strategy: list[tuple[_CountingAttr, SearchStrategy[PosArgs]]],
     frozen=False,
-) -> SearchStrategy[Tuple[Type, PosArgs, KwArgs]]:
+) -> SearchStrategy[tuple[type, PosArgs, KwArgs]]:
     """
     A helper function for Hypothesis to generate attrs classes.
 
@@ -272,9 +237,9 @@ def _create_hyp_class(
 
 
 def _create_dataclass(
-    attrs_and_strategy: List[Tuple[_CountingAttr, SearchStrategy[PosArgs]]],
+    attrs_and_strategy: list[tuple[_CountingAttr, SearchStrategy[PosArgs]]],
     frozen=False,
-) -> SearchStrategy[Tuple[Type, PosArgs, KwArgs]]:
+) -> SearchStrategy[tuple[Type, PosArgs, KwArgs]]:
     """
     A helper function for Hypothesis to generate dataclasses.
 
@@ -319,10 +284,10 @@ def _create_dataclass(
 
 
 def _create_hyp_class_and_strat(
-    attrs_and_strategy: List[Tuple[_CountingAttr, SearchStrategy[PosArg]]]
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
+    attrs_and_strategy: list[tuple[_CountingAttr, SearchStrategy[PosArg]]],
+) -> SearchStrategy[tuple[type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
     def key(t):
-        return (t[0].default is not NOTHING, t[0].kw_only)
+        return (t[0].default is not NOTHING, t[0].kw_only or False)
 
     attrs_and_strat = sorted(attrs_and_strategy, key=key)
     attrs = [a[0] for a in attrs_and_strat]
@@ -336,7 +301,7 @@ def _create_hyp_class_and_strat(
                 attr_name = attr_name[1:]
             kwarg_strats[attr_name] = attr_and_strat[1]
     return tuples(
-        just(make_class("HypClass", OrderedDict(zip(gen_attr_names(), attrs)))),
+        just(make_class("HypClass", dict(zip(gen_attr_names(), attrs)))),
         just(tuples(*vals)),
         just(fixed_dictionaries(kwarg_strats)),
     )
@@ -344,55 +309,70 @@ def _create_hyp_class_and_strat(
 
 @composite
 def any_typed_attrs(
-    draw: DrawFn, defaults=None, kw_only=None
-) -> Tuple[_CountingAttr, SearchStrategy[None]]:
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+) -> tuple[_CountingAttr, SearchStrategy[None]]:
     """Attributes typed as `Any`, having values of `None`."""
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = None
     return (
         field(
             type=Any,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         just(None),
     )
 
 
 @composite
-def int_typed_attrs(draw, defaults=None, kw_only=None):
+def int_typed_attrs(
+    draw, defaults: FeatureFlag = "sometimes", kw_only: FeatureFlag = "sometimes"
+):
     """
     Generate a tuple of an attribute and a strategy that yields ints for that
     attribute.
     """
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(integers())
     return (
         field(
             type=int,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         integers(),
     )
 
 
 @composite
-def str_typed_attrs(draw, defaults=None, kw_only=None, codec: str = "utf8"):
+def str_typed_attrs(
+    draw,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+    codec: str = "utf8",
+):
     """
     Generate a tuple of an attribute and a strategy that yields strs for that
     attribute.
     """
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(text())
     return (
         field(
             type=str,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         text(characters(codec=codec)),
     )
@@ -400,20 +380,26 @@ def str_typed_attrs(draw, defaults=None, kw_only=None, codec: str = "utf8"):
 
 @composite
 def float_typed_attrs(
-    draw, defaults=None, kw_only=None, allow_infinity=None, allow_nan=None
+    draw,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+    allow_infinity=None,
+    allow_nan=True,
 ):
     """
     Generate a tuple of an attribute and a strategy that yields floats for that
     attribute.
     """
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(floats(allow_infinity=allow_infinity, allow_nan=allow_nan))
     return (
         field(
             type=float,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         floats(allow_infinity=allow_infinity, allow_nan=allow_nan),
     )
@@ -421,22 +407,25 @@ def float_typed_attrs(
 
 @composite
 def path_typed_attrs(
-    draw: DrawFn, defaults: Optional[bool] = None, kw_only: Optional[bool] = None
-) -> Tuple[_CountingAttr, SearchStrategy[Path]]:
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+) -> tuple[_CountingAttr, SearchStrategy[Path]]:
     """
     Generate a tuple of an attribute and a strategy that yields paths for that
     attribute.
     """
-    from string import ascii_lowercase
 
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = Path(draw(text(ascii_lowercase, min_size=1)))
     return (
         field(
             type=Path,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         text(ascii_lowercase, min_size=1).map(Path),
     )
@@ -444,8 +433,11 @@ def path_typed_attrs(
 
 @composite
 def dict_typed_attrs(
-    draw, defaults=None, allow_mutable_defaults=True, kw_only=None
-) -> SearchStrategy[Tuple[_CountingAttr, SearchStrategy]]:
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    allow_mutable_defaults=True,
+    kw_only: FeatureFlag = "sometimes",
+) -> tuple[_CountingAttr, SearchStrategy[dict[str, int]]]:
     """
     Generate a tuple of an attribute and a strategy that yields dictionaries
     for that attribute. The dictionaries map strings to integers.
@@ -453,7 +445,7 @@ def dict_typed_attrs(
     """
     default = NOTHING
     val_strat = dictionaries(keys=text(), values=integers())
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -464,7 +456,9 @@ def dict_typed_attrs(
         field(
             type=type,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -472,7 +466,10 @@ def dict_typed_attrs(
 
 @composite
 def new_dict_typed_attrs(
-    draw, defaults=None, allow_mutable_defaults=True, kw_only=None
+    draw,
+    defaults: FeatureFlag = "sometimes",
+    allow_mutable_defaults=True,
+    kw_only: FeatureFlag = "sometimes",
 ):
     """
     Generate a tuple of an attribute and a strategy that yields dictionaries
@@ -482,7 +479,7 @@ def new_dict_typed_attrs(
     """
     default_val = NOTHING
     val_strat = dictionaries(keys=text(), values=integers())
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -495,7 +492,9 @@ def new_dict_typed_attrs(
         field(
             type=dict[str, int],
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -504,10 +503,10 @@ def new_dict_typed_attrs(
 @composite
 def set_typed_attrs(
     draw: DrawFn,
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     allow_mutable_defaults=True,
     legacy_types_only=False,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
 ):
     """
     Generate a tuple of an attribute and a strategy that yields sets
@@ -515,7 +514,7 @@ def set_typed_attrs(
     """
     default_val = NOTHING
     val_strat = sets(integers())
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -535,7 +534,9 @@ def set_typed_attrs(
         field(
             type=type,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -543,7 +544,10 @@ def set_typed_attrs(
 
 @composite
 def frozenset_typed_attrs(
-    draw: DrawFn, defaults=None, legacy_types_only=False, kw_only=None
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    legacy_types_only=False,
+    kw_only: FeatureFlag = "sometimes",
 ):
     """
     Generate a tuple of an attribute and a strategy that yields frozensets
@@ -551,7 +555,7 @@ def frozenset_typed_attrs(
     """
     default = NOTHING
     val_strat = frozensets(integers())
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(val_strat)
     type = draw(
         sampled_from(
@@ -564,7 +568,9 @@ def frozenset_typed_attrs(
         field(
             type=type,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -573,18 +579,18 @@ def frozenset_typed_attrs(
 @composite
 def list_typed_attrs(
     draw: DrawFn,
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     allow_mutable_defaults=True,
     legacy_types_only=False,
-    kw_only=None,
-) -> Tuple[_CountingAttr, SearchStrategy[List[float]]]:
+    kw_only: FeatureFlag = "sometimes",
+) -> tuple[_CountingAttr, SearchStrategy[list[float]]]:
     """
     Generate a tuple of an attribute and a strategy that yields lists
     for that attribute. The lists contain floats.
     """
     default_val = NOTHING
     val_strat = lists(floats(allow_infinity=False, allow_nan=False))
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -602,7 +608,9 @@ def list_typed_attrs(
                 )
             ),
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -611,18 +619,21 @@ def list_typed_attrs(
 @composite
 def seq_typed_attrs(
     draw,
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     allow_mutable_defaults=True,
     legacy_types_only=False,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
 ):
     """
-    Generate a tuple of an attribute and a strategy that yields lists
-    for that attribute. The lists contain integers.
+    Generate a tuple of an attribute and a strategy that yields tuples
+    for that attribute. The tuples contain integers.
+
+    Args:
+        allow_mutable_defaults: When false, the default will always be a factory.
     """
     default_val = NOTHING
-    val_strat = lists(integers())
-    if defaults is True or (defaults is None and draw(booleans())):
+    val_strat = lists(integers()).map(tuple)
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -635,7 +646,9 @@ def seq_typed_attrs(
         field(
             type=AbcSequence[int] if not legacy_types_only else Sequence[int],
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
@@ -644,10 +657,10 @@ def seq_typed_attrs(
 @composite
 def mutable_seq_typed_attrs(
     draw,
-    defaults=None,
+    defaults: FeatureFlag = "sometimes",
     allow_mutable_defaults=True,
     legacy_types_only=False,
-    kw_only=None,
+    kw_only: FeatureFlag = "sometimes",
 ):
     """
     Generate a tuple of an attribute and a strategy that yields lists
@@ -655,7 +668,7 @@ def mutable_seq_typed_attrs(
     """
     default_val = NOTHING
     val_strat = lists(floats(allow_infinity=False, allow_nan=False))
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default_val = draw(val_strat)
         if not allow_mutable_defaults or draw(booleans()):
             default = Factory(lambda: default_val)
@@ -672,21 +685,28 @@ def mutable_seq_typed_attrs(
                 else MutableSequence[float]
             ),
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
 
 
 @composite
-def homo_tuple_typed_attrs(draw, defaults=None, legacy_types_only=False, kw_only=None):
+def homo_tuple_typed_attrs(
+    draw,
+    defaults: FeatureFlag = "sometimes",
+    legacy_types_only=False,
+    kw_only: FeatureFlag = "sometimes",
+):
     """
     Generate a tuple of an attribute and a strategy that yields homogenous
     tuples for that attribute. The tuples contain strings.
     """
     default = NOTHING
     val_strat = tuples(text(), text(), text())
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(val_strat)
     return (
         field(
@@ -698,34 +718,46 @@ def homo_tuple_typed_attrs(draw, defaults=None, legacy_types_only=False, kw_only
                 )
             ),
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         val_strat,
     )
 
 
 @composite
-def newtype_int_typed_attrs(draw: DrawFn, defaults=None, kw_only=None):
+def newtype_int_typed_attrs(
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+):
     """
     Generate a tuple of an attribute and a strategy that yields ints for that
     attribute.
     """
     default = NOTHING
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = draw(integers())
     NewInt = NewType("NewInt", int)
     return (
         field(
             type=NewInt,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         integers(),
     )
 
 
 @composite
-def newtype_attrs_typed_attrs(draw: DrawFn, defaults=None, kw_only=None):
+def newtype_attrs_typed_attrs(
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    kw_only: FeatureFlag = "sometimes",
+):
     """
     Generate a tuple of an attribute and a strategy that yields values for that
     attribute.
@@ -736,7 +768,7 @@ def newtype_attrs_typed_attrs(draw: DrawFn, defaults=None, kw_only=None):
     class NewTypeAttrs:
         a: int
 
-    if defaults is True or (defaults is None and draw(booleans())):
+    if defaults == "always" or (defaults == "sometimes" and draw(booleans())):
         default = NewTypeAttrs(draw(integers()))
 
     NewAttrs = NewType("NewAttrs", NewTypeAttrs)
@@ -744,17 +776,19 @@ def newtype_attrs_typed_attrs(draw: DrawFn, defaults=None, kw_only=None):
         field(
             type=NewAttrs,
             default=default,
-            kw_only=draw(booleans()) if kw_only is None else kw_only,
+            kw_only=(
+                draw(booleans()) if kw_only == "sometimes" else (kw_only == "always")
+            ),
         ),
         integers().map(NewTypeAttrs),
     )
 
 
 def just_class(
-    tup: Tuple[
-        List[Tuple[_CountingAttr, SearchStrategy]], Tuple[Type, PosArgs, KwArgs]
+    tup: tuple[
+        list[tuple[_CountingAttr, SearchStrategy]], tuple[Type, PosArgs, KwArgs]
     ],
-    defaults: Tuple[PosArgs, KwArgs],
+    defaults: tuple[PosArgs, KwArgs],
 ):
     nested_cl = tup[1][0]
     nested_cl_args = tup[1][1]
@@ -771,11 +805,11 @@ def just_class(
 
 
 def list_of_class(
-    tup: Tuple[
-        List[Tuple[_CountingAttr, SearchStrategy]], Tuple[Type, PosArgs, KwArgs]
+    tup: tuple[
+        list[tuple[_CountingAttr, SearchStrategy]], tuple[type, PosArgs, KwArgs]
     ],
-    defaults: Tuple[PosArgs, KwArgs],
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
+    defaults: tuple[PosArgs, KwArgs],
+) -> SearchStrategy[tuple[type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
     nested_cl = tup[1][0]
     nested_cl_args = tup[1][1]
     nested_cl_kwargs = tup[1][2]
@@ -791,10 +825,10 @@ def list_of_class(
 
 
 def new_list_of_class(
-    tup: Tuple[
-        List[Tuple[_CountingAttr, SearchStrategy]], Tuple[Type, PosArgs, KwArgs]
+    tup: tuple[
+        list[tuple[_CountingAttr, SearchStrategy]], tuple[Type, PosArgs, KwArgs]
     ],
-    defaults: Tuple[PosArgs, KwArgs],
+    defaults: tuple[PosArgs, KwArgs],
 ):
     """Uses the new 3.9 list type annotation."""
     nested_cl = tup[1][0]
@@ -812,10 +846,10 @@ def new_list_of_class(
 
 
 def dict_of_class(
-    tup: Tuple[
-        List[Tuple[_CountingAttr, SearchStrategy]], Tuple[Type, PosArgs, KwArgs]
+    tup: tuple[
+        list[tuple[_CountingAttr, SearchStrategy]], tuple[Type, PosArgs, KwArgs]
     ],
-    defaults: Tuple[PosArgs, KwArgs],
+    defaults: tuple[PosArgs, KwArgs],
 ):
     nested_cl = tup[1][0]
     nested_cl_args = tup[1][1]
@@ -832,8 +866,11 @@ def dict_of_class(
 
 
 def _create_hyp_nested_strategy(
-    simple_class_strategy: SearchStrategy, kw_only=None, newtypes=True
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
+    simple_class_strategy: SearchStrategy,
+    kw_only: FeatureFlag = "sometimes",
+    newtypes=True,
+    allow_nan=True,
+) -> SearchStrategy[tuple[type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
     """
     Create a recursive attrs class.
     Given a strategy for building (simpler) classes, create and return
@@ -845,9 +882,10 @@ def _create_hyp_nested_strategy(
     # A strategy producing tuples of the form ([list of attributes], <given
     # class strategy>).
     attrs_and_classes: SearchStrategy[
-        Tuple[List[Tuple[_CountingAttr, PosArgs]], Tuple[Type, SearchStrategy[PosArgs]]]
+        tuple[list[tuple[_CountingAttr, PosArgs]], tuple[type, SearchStrategy[PosArgs]]]
     ] = tuples(
-        lists_of_typed_attrs(kw_only=kw_only, newtypes=newtypes), simple_class_strategy
+        lists_of_typed_attrs(kw_only=kw_only, newtypes=newtypes, allow_nan=allow_nan),
+        simple_class_strategy,
     )
 
     return nested_classes(attrs_and_classes)
@@ -857,56 +895,69 @@ def _create_hyp_nested_strategy(
 def nested_classes(
     draw: DrawFn,
     attrs_and_classes: SearchStrategy[
-        Tuple[
-            List[Tuple[_CountingAttr, SearchStrategy]],
-            Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]],
+        tuple[
+            list[tuple[_CountingAttr, SearchStrategy]],
+            tuple[type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]],
         ]
     ],
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs], SearchStrategy[KwArgs]]]:
+) -> tuple[type[AttrsInstance], SearchStrategy[PosArgs], SearchStrategy[KwArgs]]:
     attrs, class_and_strat = draw(attrs_and_classes)
     cls, strat, kw_strat = class_and_strat
     pos_defs = tuple(draw(strat))
     kwarg_defs = draw(kw_strat)
     init_vals = tuple(draw(strat))
     init_kwargs = draw(kw_strat)
-    if is_39_or_later:
-        return draw(
-            list_of_class(
-                (attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs)
-            )
-            | new_list_of_class(
-                (attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs)
-            )
-            | dict_of_class(
-                (attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs)
-            )
-            | just_class((attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs))
-        )
-
     return draw(
         list_of_class((attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs))
+        | new_list_of_class(
+            (attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs)
+        )
         | dict_of_class((attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs))
         | just_class((attrs, (cls, init_vals, init_kwargs)), (pos_defs, kwarg_defs))
     )
 
 
 def nested_typed_classes_and_strat(
-    defaults=None, min_attrs=0, kw_only=None, newtypes=True
-) -> SearchStrategy[Tuple[Type, SearchStrategy[PosArgs]]]:
+    defaults: FeatureFlag = "sometimes",
+    min_attrs=0,
+    kw_only: FeatureFlag = "sometimes",
+    newtypes=True,
+    allow_nan=True,
+) -> SearchStrategy[tuple[type, SearchStrategy[PosArgs]]]:
     return recursive(
         simple_typed_classes_and_strats(
-            defaults=defaults, min_attrs=min_attrs, kw_only=kw_only, newtypes=newtypes
+            defaults=defaults,
+            min_attrs=min_attrs,
+            kw_only=kw_only,
+            newtypes=newtypes,
+            allow_nan=allow_nan,
+        ),
+        partial(
+            _create_hyp_nested_strategy,
+            kw_only=kw_only,
+            newtypes=newtypes,
+            allow_nan=allow_nan,
         ),
-        partial(_create_hyp_nested_strategy, kw_only=kw_only, newtypes=newtypes),
         max_leaves=20,
     )
 
 
 @composite
-def nested_typed_classes(draw, defaults=None, min_attrs=0, kw_only=None, newtypes=True):
+def nested_typed_classes(
+    draw: DrawFn,
+    defaults: FeatureFlag = "sometimes",
+    min_attrs=0,
+    kw_only: FeatureFlag = "sometimes",
+    newtypes=True,
+    allow_nan=True,
+):
     cl, strat, kwarg_strat = draw(
         nested_typed_classes_and_strat(
-            defaults=defaults, min_attrs=min_attrs, kw_only=kw_only, newtypes=newtypes
+            defaults=defaults,
+            min_attrs=min_attrs,
+            kw_only=kw_only,
+            newtypes=newtypes,
+            allow_nan=allow_nan,
         )
     )
     return cl, draw(strat), draw(kwarg_strat)
