@@ -2,13 +2,12 @@
 
 import keyword
 import string
-from collections import OrderedDict
+from collections.abc import Iterable
 from enum import Enum
 from typing import (
     Any,
     Deque,
     Dict,
-    Iterable,
     List,
     Mapping,
     MutableMapping,
@@ -17,18 +16,19 @@ from typing import (
     Sequence,
     Set,
     Tuple,
-    Type,
 )
 
-import attr
+from attr import attrib
 from attr._make import _CountingAttr
 from attrs import NOTHING, AttrsInstance, Factory, make_class
 from hypothesis import strategies as st
-from hypothesis.strategies import SearchStrategy
+from hypothesis.strategies import SearchStrategy, booleans
+from typing_extensions import TypeAlias
 
 PosArg = Any
-PosArgs = Tuple[PosArg]
-KwArgs = Dict[str, Any]
+PosArgs = tuple[PosArg]
+KwArgs = dict[str, Any]
+AttrsAndArgs: TypeAlias = tuple[type[AttrsInstance], PosArgs, KwArgs]
 
 primitive_strategies = st.sampled_from(
     [
@@ -166,9 +166,9 @@ def gen_attr_names() -> Iterable[str]:
 
 
 def _create_hyp_class(
-    attrs_and_strategy: List[Tuple[_CountingAttr, st.SearchStrategy[PosArgs]]],
+    attrs_and_strategy: list[tuple[_CountingAttr, st.SearchStrategy[PosArgs]]],
     frozen=None,
-):
+) -> SearchStrategy[AttrsAndArgs]:
     """
     A helper function for Hypothesis to generate attrs classes.
 
@@ -177,7 +177,7 @@ def _create_hyp_class(
     """
 
     def key(t):
-        return (t[0].default is not NOTHING, t[0].kw_only)
+        return (t[0].default is not NOTHING, t[0].kw_only or False)
 
     attrs_and_strat = sorted(attrs_and_strategy, key=key)
     attrs = [a[0] for a in attrs_and_strat]
@@ -193,7 +193,7 @@ def _create_hyp_class(
     return st.tuples(
         st.builds(
             lambda f: make_class(
-                "HypClass", OrderedDict(zip(gen_attr_names(), attrs)), frozen=f
+                "HypClass", dict(zip(gen_attr_names(), attrs)), frozen=f
             ),
             st.booleans() if frozen is None else st.just(frozen),
         ),
@@ -204,70 +204,97 @@ def _create_hyp_class(
 
 def just_class(tup):
     nested_cl = tup[1][0]
-    default = attr.Factory(nested_cl)
+    default = Factory(nested_cl)
     combined_attrs = list(tup[0])
-    combined_attrs.append((attr.ib(default=default), st.just(nested_cl())))
+    combined_attrs.append((attrib(default=default), st.just(nested_cl())))
     return _create_hyp_class(combined_attrs)
 
 
-def just_class_with_type(tup):
+def just_class_with_type(tup: tuple) -> SearchStrategy[AttrsAndArgs]:
     nested_cl = tup[1][0]
-    default = attr.Factory(nested_cl)
-    combined_attrs = list(tup[0])
-    combined_attrs.append(
-        (attr.ib(default=default, type=nested_cl), st.just(nested_cl()))
-    )
-    return _create_hyp_class(combined_attrs)
 
+    def make_with_default(takes_self: bool) -> SearchStrategy[AttrsAndArgs]:
+        combined_attrs = list(tup[0])
+        combined_attrs.append(
+            (
+                attrib(
+                    default=(
+                        Factory(
+                            nested_cl if not takes_self else lambda _: nested_cl(),
+                            takes_self=takes_self,
+                        )
+                    ),
+                    type=nested_cl,
+                ),
+                st.just(nested_cl()),
+            )
+        )
+        return _create_hyp_class(combined_attrs)
 
-def just_class_with_type_takes_self(
-    tup: Tuple[List[Tuple[_CountingAttr, SearchStrategy]], Tuple[Type[AttrsInstance]]]
-) -> SearchStrategy[Tuple[Type[AttrsInstance]]]:
-    nested_cl = tup[1][0]
-    default = Factory(lambda _: nested_cl(), takes_self=True)
-    combined_attrs = list(tup[0])
-    combined_attrs.append(
-        (attr.ib(default=default, type=nested_cl), st.just(nested_cl()))
-    )
-    return _create_hyp_class(combined_attrs)
+    return booleans().flatmap(make_with_default)
 
 
 def just_frozen_class_with_type(tup):
     nested_cl = tup[1][0]
     combined_attrs = list(tup[0])
     combined_attrs.append(
-        (attr.ib(default=nested_cl(), type=nested_cl), st.just(nested_cl()))
+        (attrib(default=nested_cl(), type=nested_cl), st.just(nested_cl()))
     )
     return _create_hyp_class(combined_attrs)
 
 
-def list_of_class(tup):
+def list_of_class(tup: tuple) -> SearchStrategy[AttrsAndArgs]:
     nested_cl = tup[1][0]
-    default = attr.Factory(lambda: [nested_cl()])
-    combined_attrs = list(tup[0])
-    combined_attrs.append((attr.ib(default=default), st.just([nested_cl()])))
-    return _create_hyp_class(combined_attrs)
+
+    def make_with_default(takes_self: bool) -> SearchStrategy[AttrsAndArgs]:
+        combined_attrs = list(tup[0])
+        combined_attrs.append(
+            (
+                attrib(
+                    default=(
+                        Factory(lambda: [nested_cl()])
+                        if not takes_self
+                        else Factory(lambda _: [nested_cl()], takes_self=True)
+                    ),
+                    type=list[nested_cl],
+                ),
+                st.just([nested_cl()]),
+            )
+        )
+        return _create_hyp_class(combined_attrs)
+
+    return booleans().flatmap(make_with_default)
 
 
-def list_of_class_with_type(tup):
+def list_of_class_with_type(tup: tuple) -> SearchStrategy[AttrsAndArgs]:
     nested_cl = tup[1][0]
-    default = attr.Factory(lambda: [nested_cl()])
-    combined_attrs = list(tup[0])
-    combined_attrs.append(
-        (attr.ib(default=default, type=List[nested_cl]), st.just([nested_cl()]))
-    )
-    return _create_hyp_class(combined_attrs)
+
+    def make_with_default(takes_self: bool) -> SearchStrategy[AttrsAndArgs]:
+        default = (
+            Factory(lambda: [nested_cl()])
+            if not takes_self
+            else Factory(lambda _: [nested_cl()], takes_self=True)
+        )
+        combined_attrs = list(tup[0])
+        combined_attrs.append(
+            (attrib(default=default, type=List[nested_cl]), st.just([nested_cl()]))
+        )
+        return _create_hyp_class(combined_attrs)
+
+    return booleans().flatmap(make_with_default)
 
 
 def dict_of_class(tup):
     nested_cl = tup[1][0]
-    default = attr.Factory(lambda: {"cls": nested_cl()})
+    default = Factory(lambda: {"cls": nested_cl()})
     combined_attrs = list(tup[0])
-    combined_attrs.append((attr.ib(default=default), st.just({"cls": nested_cl()})))
+    combined_attrs.append((attrib(default=default), st.just({"cls": nested_cl()})))
     return _create_hyp_class(combined_attrs)
 
 
-def _create_hyp_nested_strategy(simple_class_strategy):
+def _create_hyp_nested_strategy(
+    simple_class_strategy: SearchStrategy,
+) -> SearchStrategy:
     """
     Create a recursive attrs class.
     Given a strategy for building (simpler) classes, create and return
@@ -276,6 +303,7 @@ def _create_hyp_nested_strategy(simple_class_strategy):
         * a list of simpler classes
         * a dict mapping the string "cls" to a simpler class.
     """
+
     # A strategy producing tuples of the form ([list of attributes], <given
     # class strategy>).
     attrs_and_classes = st.tuples(lists_of_attrs(defaults=True), simple_class_strategy)
@@ -287,7 +315,6 @@ def _create_hyp_nested_strategy(simple_class_strategy):
         | attrs_and_classes.flatmap(list_of_class_with_type)
         | attrs_and_classes.flatmap(dict_of_class)
         | attrs_and_classes.flatmap(just_frozen_class_with_type)
-        | attrs_and_classes.flatmap(just_class_with_type_takes_self)
     )
 
 
@@ -301,7 +328,7 @@ def bare_attrs(draw, defaults=None, kw_only=None):
     if defaults is True or (defaults is None and draw(st.booleans())):
         default = None
     return (
-        attr.ib(
+        attrib(
             default=default, kw_only=draw(st.booleans()) if kw_only is None else kw_only
         ),
         st.just(None),
@@ -318,7 +345,7 @@ def int_attrs(draw, defaults=None, kw_only=None):
     if defaults is True or (defaults is None and draw(st.booleans())):
         default = draw(st.integers())
     return (
-        attr.ib(
+        attrib(
             default=default, kw_only=draw(st.booleans()) if kw_only is None else kw_only
         ),
         st.integers(),
@@ -339,12 +366,12 @@ def str_attrs(draw, defaults=None, type_annotations=None, kw_only=None):
     else:
         type = None
     return (
-        attr.ib(
+        attrib(
             default=default,
             type=type,
             kw_only=draw(st.booleans()) if kw_only is None else kw_only,
         ),
-        st.text(),
+        st.text(max_size=5),
     )
 
 
@@ -356,9 +383,9 @@ def float_attrs(draw, defaults=None, kw_only=None):
     """
     default = NOTHING
     if defaults is True or (defaults is None and draw(st.booleans())):
-        default = draw(st.floats())
+        default = draw(st.floats(allow_nan=False))
     return (
-        attr.ib(
+        attrib(
             default=default, kw_only=draw(st.booleans()) if kw_only is None else kw_only
         ),
         st.floats(allow_nan=False),
@@ -372,12 +399,12 @@ def dict_attrs(draw, defaults=None, kw_only=None):
     for that attribute. The dictionaries map strings to integers.
     """
     default = NOTHING
-    val_strat = st.dictionaries(keys=st.text(), values=st.integers())
+    val_strat = st.dictionaries(keys=st.text(max_size=5), values=st.integers())
     if defaults is True or (defaults is None and draw(st.booleans())):
         default_val = draw(val_strat)
-        default = attr.Factory(lambda: default_val)
+        default = Factory(lambda: default_val)
     return (
-        attr.ib(
+        attrib(
             default=default, kw_only=draw(st.booleans()) if kw_only is None else kw_only
         ),
         val_strat,
@@ -396,7 +423,7 @@ def optional_attrs(draw, defaults=None, kw_only=None):
         default = draw(val_strat)
 
     return (
-        attr.ib(
+        attrib(
             default=default, kw_only=draw(st.booleans()) if kw_only is None else kw_only
         ),
         val_strat,
@@ -414,14 +441,16 @@ def simple_attrs(defaults=None, kw_only=None):
     )
 
 
-def lists_of_attrs(defaults=None, min_size=0, kw_only=None):
+def lists_of_attrs(defaults=None, min_size=0, kw_only=None) -> SearchStrategy[list]:
     # Python functions support up to 255 arguments.
     return st.lists(
         simple_attrs(defaults, kw_only), min_size=min_size, max_size=10
     ).map(lambda lst: sorted(lst, key=lambda t: t[0]._default is not NOTHING))
 
 
-def simple_classes(defaults=None, min_attrs=0, frozen=None, kw_only=None):
+def simple_classes(
+    defaults=None, min_attrs=0, frozen=None, kw_only=None
+) -> SearchStrategy:
     """
     Return a strategy that yields tuples of simple classes and values to
     instantiate them.
@@ -431,9 +460,8 @@ def simple_classes(defaults=None, min_attrs=0, frozen=None, kw_only=None):
     )
 
 
-# Ok, so st.recursive works by taking a base strategy (in this case,
-# simple_classes) and a special function. This function receives a strategy,
-# and returns another strategy (building on top of the base strategy).
-nested_classes = st.recursive(
-    simple_classes(defaults=True), _create_hyp_nested_strategy
-)
+def nested_classes() -> SearchStrategy[AttrsAndArgs]:
+    # Ok, so st.recursive works by taking a base strategy (in this case,
+    # simple_classes) and a special function. This function receives a strategy,
+    # and returns another strategy (building on top of the base strategy).
+    return st.recursive(simple_classes(defaults=True), _create_hyp_nested_strategy)
